//Port of 2011 Intern Design's RGB coffee table code to C2000 devicelib.lib
//For 2013 Intern Design Competition
//John Fenske, Richi Vyas

//TODO Calibrate math to LEDs from ADC

#include "DSP28x_Project.h"

#include "f2802x_common/include/adc.h"
#include "f2802x_common/include/clk.h"
#include "f2802x_common/include/flash.h"
#include "f2802x_common/include/gpio.h"
#include "f2802x_common/include/pie.h"
#include "f2802x_common/include/pll.h"
#include "f2802x_common/include/timer.h"
#include "f2802x_common/include/wdog.h"

#define RED_FET GPIO_Number_28
#define GRN_FET GPIO_Number_34
#define BLU_FET GPIO_Number_6

//#define XLAT    //ADCINA6
#define SIN     GPIO_Number_16
#define SCLK    GPIO_Number_18
#define BLANK   GPIO_Number_17
//#define VPRG    1 //not used
//#define GSCLK   //ADCINB6

#define RED     0
#define GRN     1
#define BLU     2

#define NUM_LEDS (12*12*3)

#define DRIVE0	GPIO_Number_0
#define DRIVE1	GPIO_Number_1
#define DRIVE2	GPIO_Number_2
#define DRIVE3	GPIO_Number_3
#define DRIVE4	GPIO_Number_4
#define DRIVE5	GPIO_Number_5
#define DRIVE6	GPIO_Number_32
#define DRIVE7	GPIO_Number_33
#define DRIVE8	GPIO_Number_19
#define DRIVE9	GPIO_Number_12
#define DRIVE10	GPIO_Number_7

// Prototype statements for functions found within this file.
interrupt void cpu_timer0_isr(void);
interrupt void adc_isr(void);



ADC_Handle myAdc;
CLK_Handle myClk;
FLASH_Handle myFlash;
GPIO_Handle myGpio;
PIE_Handle myPie;
PWM_Handle myPwm;
TIMER_Handle myTimer;

// [R0, G0, B0, R1, G1, B1, R2, G2, ...]
uint16_t table[NUM_LEDS]; //using int to hold ADC values, although only 8 bits are used for LED driving
uint16_t table2[NUM_LEDS]; //remapped values

uint16_t color = RED;

uint16_t need_xlat = 0;

uint16_t need_extra_sclk = 0;

uint16_t i;
uint16_t GSCLKcounter = 0;
uint16_t driveLine = 0;
uint16_t testwatch = 0;
uint16_t ConversionCount = 0;

static inline void init_table()
{
    unsigned char r,g,b;

    for (i=0; i<NUM_LEDS; i+=3)
    {
        if (i < (1*12*3) ) {r = 255; g = 000; b = 000;}
        else if (i < (2*12*3) ) {r = 000; g = 255; b = 000;}
        else if (i < (3*12*3) ) {r = 000; g = 000; b = 255;}
        else if (i < (4*12*3) ) {r = 255; g = 255; b = 000;}
        else if (i < (5*12*3) ) {r = 255; g = 000; b = 255;}
        else if (i < (6*12*3) ) {r = 000; g = 255; b = 255;}
        else if (i < (12*12*3) ) {r = 255; g = 255; b = 255;}
        else if (i < (12*12*3) ) {r = 255; g = 255; b = 255;}

        table[i+RED] = r;
        table[i+GRN] = g;
        table[i+BLU] = b;
    }

}

static inline void next_color()
{
    switch (color)
    {
        case RED:
        	GPIO_setHigh(myGpio, GRN_FET);
        	GPIO_setHigh(myGpio, BLU_FET);
            GPIO_setLow(myGpio, RED_FET);
            color = GRN;
            break;
        case GRN:
        	GPIO_setHigh(myGpio, RED_FET);
        	GPIO_setHigh(myGpio, BLU_FET);
            GPIO_setLow(myGpio, GRN_FET);
            color = BLU;
            break;
        case BLU:
        	GPIO_setHigh(myGpio, RED_FET);
        	GPIO_setHigh(myGpio, GRN_FET);
            GPIO_setLow(myGpio, BLU_FET);
            color = RED;
            break;
        default:
        	GPIO_setHigh(myGpio, RED_FET);
        	GPIO_setHigh(myGpio, GRN_FET);
        	GPIO_setHigh(myGpio, BLU_FET);
            color = RED;
            break;
    }
}

static inline void fets_off()
{
	GPIO_setHigh(myGpio, RED_FET);
	GPIO_setHigh(myGpio, GRN_FET);
	GPIO_setHigh(myGpio, BLU_FET);
}

static inline void refresh_table()
{
    // if were waiting for a latch dont update again
    if (need_xlat == 0)
    {
        // shift in all the values for the current color
        for (i=0; i<NUM_LEDS; i+=3)
        {
            // unrolled loops for speed

            // 4 pad bits
        	GPIO_setLow(myGpio, SIN);

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (color == RED)
            {
            	//probably need to scale ADC values here
            	//table[i+color] = scaleFactor1*(table[i+color] - scaleFactor2);
            	//or use threshold
            	//if (table[i + color] > threshold)
            	//{
            	//	table[i + color] = 255;
            	//}
            	//else
            	//{
            	//	table[i + color] = 0;
            	//}
            }

            // 8 data bits
            if (table[i+color] & (1 << 7)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 6)) {GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 5)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 4)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 3)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 2)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 1)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);

            if (table[i+color] & (1 << 0)) { GPIO_setHigh(myGpio, SIN); } else { GPIO_setLow(myGpio, SIN); }

            GPIO_setHigh(myGpio, SCLK);
            GPIO_setLow(myGpio, SCLK);
        }

        // we just updated the shift registers, we need a latch
        need_xlat = 1;
    }
}

int main(void)
{

    CPU_Handle myCpu;
    PLL_Handle myPll;
    WDOG_Handle myWDog;

    // Initialize all the handles needed for this application
    myAdc = ADC_init((void *)ADC_BASE_ADDR, sizeof(ADC_Obj));
    myClk = CLK_init((void *)CLK_BASE_ADDR, sizeof(CLK_Obj));
    myCpu = CPU_init((void *)NULL, sizeof(CPU_Obj));
    myFlash = FLASH_init((void *)FLASH_BASE_ADDR, sizeof(FLASH_Obj));
    myGpio = GPIO_init((void *)GPIO_BASE_ADDR, sizeof(GPIO_Obj));
    myPie = PIE_init((void *)PIE_BASE_ADDR, sizeof(PIE_Obj));
    myPll = PLL_init((void *)PLL_BASE_ADDR, sizeof(PLL_Obj));
    myPwm = PWM_init((void *)PWM_ePWM1_BASE_ADDR, sizeof(PWM_Obj));
    myTimer = TIMER_init((void *)TIMER0_BASE_ADDR, sizeof(TIMER_Obj));
    myWDog = WDOG_init((void *)WDOG_BASE_ADDR, sizeof(WDOG_Obj));
    //
    // clock
    //

    // Perform basic system initialization
    WDOG_disable(myWDog);
    CLK_enableAdcClock(myClk);
    CLK_enableTbClockSync(myClk);
    CLK_enablePwmClock(myClk, PWM_Number_1);
    (*Device_cal)();

    //Select the internal oscillator 1 as the clock source
    CLK_setOscSrc(myClk, CLK_OscSrc_Internal);
    // Setup the PLL for x10 /2 which will yield 50Mhz = 10Mhz * 10 / 2
    PLL_setup(myPll, PLL_Multiplier_10, PLL_DivideSelect_ClkIn_by_2);

    // Disable the PIE and all interrupts
    PIE_disable(myPie);
    PIE_disableAllInts(myPie);
    CPU_disableGlobalInts(myCpu);
    CPU_clearIntFlags(myCpu);

    // Setup a debug vector table and enable the PIE
    PIE_setDebugIntVectorTable(myPie);
    PIE_enable(myPie);

    // Register interrupt handlers in the PIE vector table
    PIE_registerPieIntHandler(myPie, PIE_GroupNumber_1, PIE_SubGroupNumber_7, (intVec_t)&cpu_timer0_isr);
    PIE_registerPieIntHandler(myPie, PIE_GroupNumber_10, PIE_SubGroupNumber_1, (intVec_t)&adc_isr);

    // If running from flash copy RAM only functions to RAM
    #ifdef _FLASH
        memcpy(&RamfuncsRunStart, &RamfuncsLoadStart, (size_t)&RamfuncsLoadSize);
    #endif

    // Initialize the ADC
    ADC_enableBandGap(myAdc);
    ADC_enableRefBuffers(myAdc);
    ADC_powerUp(myAdc);
    ADC_enable(myAdc);
    ADC_setVoltRefSrc(myAdc, ADC_VoltageRefSrc_Int);

    // Configure ADC
    //Note: Channel ADCINA4  will be double sampled to workaround the ADC 1st sample issue for rev0 silicon errata
    ADC_setIntPulseGenMode(myAdc, ADC_IntPulseGenMode_Prior);               //ADCINT1 trips after AdcResults latch
    ADC_enableInt(myAdc, ADC_IntNumber_1);                                  //Enabled ADCINT1
    ADC_setIntMode(myAdc, ADC_IntNumber_1, ADC_IntMode_ClearFlag);          //Disable ADCINT1 Continuous mode
    ADC_setIntSrc(myAdc, ADC_IntNumber_1, ADC_IntSrc_EOC10);                 //setup EOC2 to trigger ADCINT1 to fire
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_0, ADC_SocChanNumber_A0);    //set SOC0 channel select to ADCINA0
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_1, ADC_SocChanNumber_A1);    //set SOC1 channel select to ADCINA1
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_2, ADC_SocChanNumber_A2);    //set SOC2 channel select to ADCINA2
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_3, ADC_SocChanNumber_A3);    //set SOC2 channel select to ADCINA3
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_4, ADC_SocChanNumber_A4);    //set SOC2 channel select to ADCINA4
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_5, ADC_SocChanNumber_A7);    //set SOC2 channel select to ADCINA7
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_6, ADC_SocChanNumber_B1);    //set SOC2 channel select to ADCINB1
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_7, ADC_SocChanNumber_B2);    //set SOC2 channel select to ADCINB2
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_8, ADC_SocChanNumber_B3);    //set SOC2 channel select to ADCINB3
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_9, ADC_SocChanNumber_B4);    //set SOC2 channel select to ADCINB4
    ADC_setSocChanNumber (myAdc, ADC_SocNumber_10, ADC_SocChanNumber_B7);    //set SOC2 channel select to ADCINB7
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_0, ADC_SocTrigSrc_EPWM1_ADCSOCA);    //set SOC0 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_1, ADC_SocTrigSrc_EPWM1_ADCSOCA);    //set SOC1 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_2, ADC_SocTrigSrc_EPWM1_ADCSOCA);    //set SOC2 start trigger on EPWM1A, due to round-robin SOC0 converts first then SOC1, then SOC2
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_3, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_4, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_5, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_6, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_7, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_8, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_9, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocTrigSrc(myAdc, ADC_SocNumber_10, ADC_SocTrigSrc_EPWM1_ADCSOCA);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_0, ADC_SocSampleWindow_7_cycles);   //set SOC0 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_1, ADC_SocSampleWindow_7_cycles);   //set SOC1 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_2, ADC_SocSampleWindow_7_cycles);   //set SOC2 S/H Window to 7 ADC Clock Cycles, (6 ACQPS plus 1)
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_3, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_4, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_5, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_6, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_7, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_8, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_9, ADC_SocSampleWindow_7_cycles);
    ADC_setSocSampleWindow(myAdc, ADC_SocNumber_10, ADC_SocSampleWindow_7_cycles);

    // Enable PWM clock
    CLK_enablePwmClock(myClk, PWM_Number_1);

    // Setup PWM
    PWM_enableSocAPulse(myPwm);                                         // Enable SOC on A group
    PWM_setSocAPulseSrc(myPwm, PWM_SocPulseSrc_CounterEqualCmpAIncr);   // Select SOC from from CPMA on upcount
    PWM_setSocAPeriod(myPwm, PWM_SocPeriod_FirstEvent);                 // Generate pulse on 1st event
    PWM_setCmpA(myPwm, 0x0400);                                         // Set compare A value
    PWM_setPeriod(myPwm, 0x0800);                                       // Set period for ePWM1 //32 samples and 11 drive lines = ~70Hz update rate
    PWM_setCounterMode(myPwm, PWM_CounterMode_Up);                      // count up and start

    //GPIO init here

    //Drive Lines

    GPIO_setMode(myGpio, GPIO_Number_0, GPIO_0_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_0, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_1, GPIO_1_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_1, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_2, GPIO_2_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_2, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_3, GPIO_3_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_3, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_4, GPIO_4_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_4, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_5, GPIO_5_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_5, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_32, GPIO_32_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_32, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_33, GPIO_33_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_33, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_19, GPIO_19_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_19, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_12, GPIO_12_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_12, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_7, GPIO_7_Mode_GeneralPurpose);
    GPIO_setDirection(myGpio, GPIO_Number_7, GPIO_Direction_Output);

    //
    // FETS
    //

    GPIO_setMode(myGpio, GPIO_Number_28, GPIO_28_Mode_GeneralPurpose); //RED_FET
    GPIO_setDirection(myGpio, GPIO_Number_28, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_34, GPIO_34_Mode_GeneralPurpose); //GRN_FET
    GPIO_setDirection(myGpio, GPIO_Number_34, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_6, GPIO_6_Mode_GeneralPurpose); //BLU_FET
    GPIO_setDirection(myGpio, GPIO_Number_6, GPIO_Direction_Output);

    fets_off();

    //
    // TLC5941
    //
EALLOW;
    // all control lines are outputs
    GpioCtrlRegs.AIOMUX1.bit.AIO6 = 0; //XLAT, ADCINA6
    GpioCtrlRegs.AIODIR.bit.AIO6 = 1;
    GPIO_setMode(myGpio, GPIO_Number_16, GPIO_16_Mode_GeneralPurpose); //SIN
    GPIO_setDirection(myGpio, GPIO_Number_16, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_18, GPIO_18_Mode_GeneralPurpose); //SCLK
    GPIO_setDirection(myGpio, GPIO_Number_18, GPIO_Direction_Output);
    GPIO_setMode(myGpio, GPIO_Number_17, GPIO_17_Mode_GeneralPurpose); //BLANK
    GPIO_setDirection(myGpio, GPIO_Number_17, GPIO_Direction_Output);
    GpioCtrlRegs.AIOMUX1.bit.AIO14 = 0; //GSCLK, ADCINB6
    GpioCtrlRegs.AIODIR.bit.AIO14 = 1;
EDIS;

    // set all control lines to 0
EALLOW;
    GpioDataRegs.AIODAT.bit.AIO6 = 0; //ADC I/O, XLAT
    GPIO_setLow(myGpio, SIN);
    GPIO_setLow(myGpio, SCLK);
    GPIO_setLow(myGpio, BLANK);
    GpioDataRegs.AIODAT.bit.AIO14 = 0; //ADC I/O, GSCLK
    EDIS;

    // setup timers //used for GSCLK
    // Configure CPU-Timer 0 to interrupt every 500 milliseconds:
    // 60MHz CPU Freq, 50 millisecond Period (in uSeconds)
    //    ConfigCpuTimer(&CpuTimer0, 60, 500000);
    //Image Refresh Rate want more than 60Hz*3 colors * 256 grayscale bits
    TIMER_stop(myTimer);
    TIMER_setPeriod(myTimer, 2500); //should be fast enough, about 78Hz update for XLAT
    TIMER_setPreScaler(myTimer, 0);
    TIMER_reload(myTimer);
    TIMER_setEmulationMode(myTimer, TIMER_EmulationMode_StopAfterNextDecrement);
    TIMER_enableInt(myTimer);

    TIMER_start(myTimer);

    // enable interrupts
    // Enable CPU INT1 which is connected to CPU-Timer 0:
    CPU_enableInt(myCpu, CPU_IntNumber_1);
    // Enable TINT0 in the PIE: Group 1 interrupt 7
    PIE_enableTimer0Int(myPie);
    // Enable ADCINT1 in PIE
    PIE_enableAdcInt(myPie, ADC_IntNumber_1);
    // Enable CPU Interrupt 1
    CPU_enableInt(myCpu, CPU_IntNumber_10);
    // Enable Global interrupt INTM
    CPU_enableGlobalInts(myCpu);
    // Enable Global realtime interrupt DBGM
    CPU_enableDebugInt(myCpu);

    // initialize the table array
    init_table();

    while(1)
    {
       refresh_table();
    }

    return 0;
}

interrupt void cpu_timer0_isr(void)
{
	EALLOW;
	GpioDataRegs.AIODAT.bit.AIO14 = ~GpioDataRegs.AIODAT.bit.AIO14; //Toggle GSCLK
	EDIS;
	GSCLKcounter++;
	if (GSCLKcounter == 256) //8 bit grayscale
	{
		GSCLKcounter = 0;
		 // pull BLANK high to reset the GS counter
		GPIO_setHigh(myGpio, BLANK);

		// check if we have new data that needs to be latched
		if (need_xlat == 1)
		{
			// latch all the new data
			EALLOW;
			GpioDataRegs.AIODAT.bit.AIO6 = 1; //ADC I/O, XLAT
			GpioDataRegs.AIODAT.bit.AIO6 = 0; //ADC I/O, XLAT
			EDIS;
			if (color == BLU)
			{
				need_xlat = 2; //set to 0 in ADC for new data
			}
			next_color();

			if (need_extra_sclk)
			{
				GPIO_setHigh(myGpio, SCLK);
				GPIO_setLow(myGpio, SCLK);
				need_extra_sclk = 0;
			}
		}

		// enable the GS counter
		GPIO_setLow(myGpio, BLANK);

	}
    // Acknowledge this interrupt to receive more interrupts from group 1
    PIE_clearInt(myPie, PIE_GroupNumber_1);
}

interrupt void adc_isr(void)
{
	if (testwatch == 1)
	{
		if (need_xlat == 2)
		{
			testwatch = 0;
			for (i=0;i<NUM_LEDS;i++)
			{
				table[i] = 0;
			}
		}
	    // Clear ADCINT1 flag reinitialize for next SOC
	    ADC_clearIntFlag(myAdc, ADC_IntNumber_1);
	    // Acknowledge interrupt to PIE
	    PIE_clearInt(myPie, PIE_GroupNumber_10);
	    return;
	}

	if (ConversionCount == 32)
	{
		ConversionCount = 0;
		driveLine++;
		if (driveLine == 13)
		{
			need_xlat = 0;
			testwatch = 1;
			driveLine = 0;
			// Clear ADCINT1 flag reinitialize for next SOC
			ADC_clearIntFlag(myAdc, ADC_IntNumber_1);
			// Acknowledge interrupt to PIE
			PIE_clearInt(myPie, PIE_GroupNumber_10);
			return;
		}
	}

	if ((ConversionCount & 0x0003) == 0)
	{
		switch (driveLine)
		{
			case 0:
				GPIO_setHigh(myGpio, DRIVE0);
				break;
			case 1:
				GPIO_setHigh(myGpio, DRIVE1);
				break;
			case 2:
				GPIO_setHigh(myGpio, DRIVE2);
				break;
			case 3:
				GPIO_setHigh(myGpio, DRIVE3);
				break;
			case 4:
				GPIO_setHigh(myGpio, DRIVE4);
				break;
			case 5:
				GPIO_setHigh(myGpio, DRIVE5);
				break;
			case 6:
				GPIO_setHigh(myGpio, DRIVE6);
				break;
			case 7:
				GPIO_setHigh(myGpio, DRIVE7);
				break;
			case 8:
				GPIO_setHigh(myGpio, DRIVE8);
				break;
			case 9:
				GPIO_setHigh(myGpio, DRIVE9);
				break;
			case 10:
				GPIO_setHigh(myGpio, DRIVE10);
				break;
		}

	}
	else if ((ConversionCount & 0x0003) == 1)
	{
		//take sum of 8 values
		//just modify red for now
		table[0*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_0);
		table[1*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_1);
		table[2*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_2);
		table[3*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_3);
		table[4*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_4);
		table[5*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_5);
		table[6*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_6);
		table[7*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_7);
		table[8*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_8);
		table[9*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_9);
		table[10*3 + 11*driveLine*3] += ADC_readResult(myAdc, ADC_ResultNumber_10);
		//The outer rows will be defined only once in table_init()
		//An improvement would be to interpolate these over the LED matrix
	}
	else if ((ConversionCount & 0x0003) == 2)
	{
		switch (driveLine)
		{
			case 0:
				GPIO_setLow(myGpio, DRIVE0);
				break;
			case 1:
				GPIO_setLow(myGpio, DRIVE1);
				break;
			case 2:
				GPIO_setLow(myGpio, DRIVE2);
				break;
			case 3:
				GPIO_setLow(myGpio, DRIVE3);
				break;
			case 4:
				GPIO_setLow(myGpio, DRIVE4);
				break;
			case 5:
				GPIO_setLow(myGpio, DRIVE5);
				break;
			case 6:
				GPIO_setLow(myGpio, DRIVE6);
				break;
			case 7:
				GPIO_setLow(myGpio, DRIVE7);
				break;
			case 8:
				GPIO_setLow(myGpio, DRIVE8);
				break;
			case 9:
				GPIO_setLow(myGpio, DRIVE9);
				break;
			case 10:
				GPIO_setLow(myGpio, DRIVE10);
				break;
		}
	}

	//Ignore 4th rotation

	ConversionCount++;
    // Clear ADCINT1 flag reinitialize for next SOC
    ADC_clearIntFlag(myAdc, ADC_IntNumber_1);
    // Acknowledge interrupt to PIE
    PIE_clearInt(myPie, PIE_GroupNumber_10);

    return;
}
void tableRemap(void)
{
	//LEDs not wired in exact order
	//Fixing order in software for easy addressing
	for (i=0;i<3;i++)
	{
	table2[0+i] = table[0+i];
	table2[1+i] = table[1+i];
	table2[2+i] = table[2+i];
	table2[3+i] = table[3+i];
	table2[4+i] = table[4+i];
	table2[5+i] = table[5+i];
	table2[6+i] = table[6+i];
	table2[7+i] = table[7+i];
	table2[8+i] = table[8+i];
	table2[9+i] = table[9+i];
	table2[10+i] = table[10+i];
	table2[11+i] = table[11+i];
	table2[12+i] = table[12+i];
	table2[13+i] = table[13+i];
	table2[14+i] = table[14+i];
	table2[15+i] = table[15+i];
	table2[16+i] = table[16+i];
	table2[17+i] = table[17+i];
	table2[18+i] = table[18+i];
	table2[19+i] = table[19+i];
	table2[20+i] = table[20+i];
	table2[21+i] = table[21+i];
	table2[22+i] = table[22+i];
	table2[23+i] = table[23+i];
	table2[24+i] = table[24+i];
	table2[25+i] = table[25+i];
	table2[26+i] = table[26+i];
	table2[27+i] = table[27+i];
	table2[28+i] = table[28+i];
	table2[29+i] = table[29+i];
	table2[30+i] = table[30+i];
	table2[31+i] = table[31+i];
	table2[32+i] = table[32+i];
	table2[33+i] = table[33+i];
	table2[34+i] = table[34+i];
	table2[35+i] = table[35+i];
	table2[36+i] = table[36+i];
	table2[37+i] = table[37+i];
	table2[38+i] = table[38+i];
	table2[39+i] = table[39+i];
	table2[40+i] = table[40+i];
	table2[41+i] = table[41+i];
	table2[42+i] = table[42+i];
	table2[43+i] = table[43+i];
	table2[44+i] = table[44+i];
	table2[45+i] = table[45+i];
	table2[46+i] = table[46+i];
	table2[47+i] = table[47+i];
	table2[48+i] = table[48+i];
	table2[49+i] = table[49+i];
	table2[50+i] = table[50+i];
	table2[51+i] = table[51+i];
	table2[52+i] = table[52+i];
	table2[53+i] = table[53+i];
	table2[54+i] = table[54+i];
	table2[55+i] = table[55+i];
	table2[56+i] = table[56+i];
	table2[57+i] = table[57+i];
	table2[58+i] = table[58+i];
	table2[59+i] = table[59+i];
	table2[60+i] = table[60+i];
	table2[61+i] = table[61+i];
	table2[62+i] = table[62+i];
	table2[63+i] = table[63+i];
	table2[64+i] = table[64+i];
	table2[65+i] = table[65+i];
	table2[66+i] = table[66+i];
	table2[67+i] = table[67+i];
	table2[68+i] = table[68+i];
	table2[69+i] = table[69+i];
	table2[70+i] = table[70+i];
	table2[71+i] = table[71+i];
	table2[72+i] = table[72+i];
	table2[73+i] = table[73+i];
	table2[74+i] = table[74+i];
	table2[75+i] = table[75+i];
	table2[76+i] = table[76+i];
	table2[77+i] = table[77+i];
	table2[78+i] = table[78+i];
	table2[79+i] = table[79+i];
	table2[80+i] = table[80+i];
	table2[81+i] = table[81+i];
	table2[82+i] = table[82+i];
	table2[83+i] = table[83+i];
	table2[84+i] = table[84+i];
	table2[85+i] = table[85+i];
	table2[86+i] = table[86+i];
	table2[87+i] = table[87+i];
	table2[88+i] = table[88+i];
	table2[89+i] = table[89+i];
	table2[90+i] = table[90+i];
	table2[91+i] = table[91+i];
	table2[92+i] = table[92+i];
	table2[93+i] = table[93+i];
	table2[94+i] = table[94+i];
	table2[95+i] = table[95+i];
	table2[96+i] = table[96+i];
	table2[97+i] = table[97+i];
	table2[98+i] = table[98+i];
	table2[99+i] = table[99+i];
	table2[100+i] = table[100+i];
	table2[101+i] = table[101+i];
	table2[102+i] = table[102+i];
	table2[103+i] = table[103+i];
	table2[104+i] = table[104+i];
	table2[105+i] = table[105+i];
	table2[106+i] = table[106+i];
	table2[107+i] = table[107+i];
	table2[108+i] = table[108+i];
	table2[109+i] = table[109+i];
	table2[110+i] = table[110+i];
	table2[111+i] = table[111+i];
	table2[112+i] = table[112+i];
	table2[113+i] = table[113+i];
	table2[114+i] = table[114+i];
	table2[115+i] = table[115+i];
	table2[116+i] = table[116+i];
	table2[117+i] = table[117+i];
	table2[118+i] = table[118+i];
	table2[119+i] = table[119+i];
	table2[120+i] = table[120+i];
	table2[121+i] = table[121+i];
	table2[122+i] = table[122+i];
	table2[123+i] = table[123+i];
	table2[124+i] = table[124+i];
	table2[125+i] = table[125+i];
	table2[126+i] = table[126+i];
	table2[127+i] = table[127+i];
	table2[128+i] = table[128+i];
	table2[129+i] = table[129+i];
	table2[130+i] = table[130+i];
	table2[131+i] = table[131+i];
	table2[132+i] = table[132+i];
	table2[133+i] = table[133+i];
	table2[134+i] = table[134+i];
	table2[135+i] = table[135+i];
	table2[136+i] = table[136+i];
	table2[137+i] = table[137+i];
	table2[138+i] = table[138+i];
	table2[139+i] = table[139+i];
	table2[140+i] = table[140+i];
	table2[141+i] = table[141+i];
	table2[142+i] = table[142+i];
	table2[143+i] = table[143+i];
	}
}




